//+------------------------------------------------------------------+
//|                                                 MarketOrdersOHLC |
//|                                                      nicholishen |
//|                                                         nick.com |
//+------------------------------------------------------------------+
#include <arrays/list.mqh>
#include <generic/hashmap.mqh>

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
datetime barTime(long time_msc, int period_seconds)
{
   long time_in_seconds = time_msc / 1000;
   datetime bar_time = datetime(floor(time_in_seconds / period_seconds) * period_seconds);
   return bar_time;
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class VolBar : public CObject
{
public:
   MqlRates bar;
   VolBar()
   {
      ZeroMemory(this.bar);
   }
   virtual int       Compare(const CObject *node, const int mode=0) const override
   {
      const VolBar *other = node;
      if (this.bar.time > other.bar.time)
         return mode == 0 ? 1 : -1;
      if (this.bar.time < other.bar.time)
         return mode == 0 ? -1 : 1;
      return 0;
   }
};
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class Volumizer : public CObject
{
   CList             m_garbage_collector;
   CHashMap<datetime, VolBar*>  m_barmap;
   VolBar            *m_volbar;
   long              m_last_tick_synced_time;
   double            m_working_volume;
   long              m_time_begin;
   ENUM_TIMEFRAMES   m_timeframe;
   int               m_period_seconds;
   double            m_buy_vol;
   double            m_sell_vol;
   long              m_ticks_processed;


public:
   Volumizer();
   ~Volumizer();
   CHashMap<datetime, VolBar*>*    initialize(long from_time_msc, ENUM_TIMEFRAMES tf=PERIOD_CURRENT);
   CHashMap<datetime, VolBar*>*    refresh(long from_time_msc=LONG_MIN,bool is_init=false);

};

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Volumizer::Volumizer(void)
{
   m_barmap.Clear();
   m_garbage_collector.Clear();
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
Volumizer::~Volumizer()
{

   m_barmap.Clear();
   m_garbage_collector.Clear();

}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CHashMap<datetime, VolBar*>* Volumizer::initialize(long from_time_msc, ENUM_TIMEFRAMES tf=PERIOD_CURRENT)
{
   m_working_volume = 0.0;
   m_volbar = NULL;
   m_time_begin = from_time_msc;
   m_timeframe = tf;
   m_period_seconds = PeriodSeconds(tf);
   m_barmap.Clear();
   m_garbage_collector.Clear();
   m_buy_vol = 0.0;
   m_sell_vol = 0.0;
   m_ticks_processed = 0;
   return this.refresh(from_time_msc, true);
}


CHashMap<datetime, VolBar*>* Volumizer::refresh(long from_time_msc=LONG_MIN, bool is_init=false)
{
   MqlTick ticks[];
   if (from_time_msc == LONG_MIN)
      from_time_msc = m_last_tick_synced_time;
   long to_time = TimeCurrent() * 1000;
   //datetime to_bar_time_debug = barTime(to_time, m_period_seconds);

   int num_ticks = CopyTicksRange(_Symbol, ticks, COPY_TICKS_TRADE, from_time_msc, to_time);
   for (int i=0; i<num_ticks; i++)
   {
      MqlTick t = ticks[i];
      if (t.time_msc <= from_time_msc)
         continue;
      if (is_init)
         m_ticks_processed++;
      datetime bar_time = barTime(t.time_msc, m_period_seconds);
      //if (bar_time == to_bar_time_debug)
      //   DebugBreak();

      double tick_net_volume = 0.0;
      if ((t.flags&TICK_FLAG_BUY)==TICK_FLAG_BUY)
      {
         tick_net_volume = t.volume_real;
         if (is_init)
            m_buy_vol += tick_net_volume;
      }
      else if ((t.flags&TICK_FLAG_SELL)==TICK_FLAG_SELL)
      {
         tick_net_volume = -t.volume_real;
         if (is_init)
            m_sell_vol += tick_net_volume;
      }
      else
      {
         continue;
      }
      m_last_tick_synced_time = t.time_msc;
      if (!CheckPointer(m_volbar) || bar_time > m_volbar.bar.time)
      {
         m_volbar = new VolBar();
         m_garbage_collector.Add(m_volbar);
         m_barmap.Add(bar_time, m_volbar);
         m_volbar.bar.time = bar_time;
         m_volbar.bar.open = m_working_volume;
         m_volbar.bar.high = m_working_volume;
         m_volbar.bar.low = m_working_volume;
         m_volbar.bar.close = m_working_volume;
      }
      m_volbar.bar.close += tick_net_volume;
      m_working_volume = m_volbar.bar.close;
      if (m_volbar.bar.close > m_volbar.bar.high)
      {
         m_volbar.bar.high = m_volbar.bar.close;
      }
      if (m_volbar.bar.close < m_volbar.bar.low)
      {
         m_volbar.bar.low = m_volbar.bar.close;
      }
   }
   m_working_volume = m_volbar.bar.close;
   if (is_init)
   {
      printf("ticks processed = %d, buy vol = %.1f, sell vol = %.1f,",
             m_ticks_processed,
             m_buy_vol,
             m_sell_vol
            );
   }
   return &m_barmap;
}
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
